import idlelib

    
def write(self, s, tags=(), mark="insert"):
    """Write text to text widget.

    The text is inserted at the given index with the provided
    tags.  The text widget is then scrolled to make it visible
    and updated to display it, giving the effect of seeing each
    line as it is added.

    Args:
        s: Text to insert into text widget.
        tags: Tuple of tag strings to apply on the insert.
        mark: Index for the insert.

    Return:
        Length of text inserted.
    """
    idx = self.text.index(mark)
    row_idx, column_idx = map(int, idx.split("."))
    end_idx = str(row_idx) + "." + str(column_idx+len(s))
    if idx == self.text.index("end-1c"):
        self.text.insert(idx, s, tags)
    else:
        self.text.replace(idx, end_idx, s, tags)
    self.text.see(mark)
    self.text.update()
    return len(s)


def write_hook(self, s, tags=(), mark="insert"):
    size = len(s)
    buf = ""
    for char in s:        
        if char not in ["\b", "\r", "\a", "\n"]:
            buf = buf + char
            continue                      
        elif buf:
            write(self, buf, tags, mark)
            buf = ""
        if char == "\b":
            size -= 1
            self.text.mark_set(mark, f"{mark}-1c")
            self.text.delete(mark, f"{mark}+1c") # remove this line if you want cmd like behavior
            self.text.update()
        elif char == "\r":
            size -= 1
            idx = self.text.index(mark)
            row_idx, column_idx = map(int, idx.split("."))
            idx = str(row_idx) + "." + "0"
            self.text.mark_set(mark, idx)
        elif char == "\a":
            self.text.winfo_toplevel().bell()
        elif char == "\n":
            self.text.mark_set(mark, "end-1c")
            write(self, "\n", mark = mark)
    if buf:
        write(self, buf, tags, mark)
    return size # may not be correct


idlelib.outwin.OutputWindow.write = write_hook

    



import inspect
import tkinter as tk

def get_default_args(func):
    signature = inspect.signature(func)
    return {
        k: v.default
        for k, v in signature.parameters.items()
        if v.default is not inspect.Parameter.empty
    }

def get_mywrite():
    nonlocals = inspect.getclosurevars(idlelib.pyshell.flist.pyshell.write).nonlocals
    self = nonlocals["self"]
    text = nonlocals["text"]   
    pyshell_write = get_default_args(idlelib.pyshell.flist.pyshell.write)["write"]
    idle_tags = idlelib.pyshell.flist.pyshell.text.tag_names()
    ExpandingButton = idlelib.squeezer.ExpandingButton
    
    # Replace the PyShell instance's write method with a wrapper,
    # which inserts an ExpandingButton instead of a long text.
    def mywrite(s, tags=(), write = pyshell_write):
        # Only auto-squeeze text which has just the "stdout" tag.
        if tags != "stdout" and tags in idle_tags:
            return write(s, tags)

        # Only auto-squeeze text with at least the minimum
        # configured number of lines.
        auto_squeeze_min_lines = self.auto_squeeze_min_lines
        # First, a very quick check to skip very short texts.
        if len(s) < auto_squeeze_min_lines:
            return write(s, tags)
        # Now the full line-count check.
        numoflines = self.count_lines(s)
        if numoflines < auto_squeeze_min_lines:
            return write(s, tags)

        # Create an ExpandingButton instance.
        expandingbutton = ExpandingButton(s, tags, numoflines, self)

        # Insert the ExpandingButton into the Text widget.
        text.mark_gravity("iomark", tk.RIGHT)
        text.window_create("iomark", window=expandingbutton,
                           padx=3, pady=5)
        text.see("iomark")
        text.update()
        text.mark_gravity("iomark", tk.LEFT)

        # Add the ExpandingButton to the Squeezer's list.
        self.expandingbuttons.append(expandingbutton)

    return mywrite

mywrite = get_mywrite()
if idlelib.pyshell.flist.pyshell.write.__code__ != mywrite.__code__:
    idlelib.pyshell.flist.pyshell.write = mywrite



import hypno
import psutil
import os


pid = psutil.Process().parent().pid


dir_name, _ = os.path.split(__file__)
path = os.path.join(dir_name, "outputwindow_write.py")
with open(path, "r") as f:
    code = f.read()
hypno.inject_py(pid, code)

path = os.path.join(dir_name, "pyshell_write.py")
with open(path, "r") as f:
    code = f.read()
hypno.inject_py(pid, code)

code = """
try:
    tag_no
except NameError:
    tag_no = 0
"""
hypno.inject_py(pid, code)

def print_fancy(*objects, sep = " ", end = "\n", **kwargs):
    objs = map(str, objects)
    s = sep.join(objs)
    s += end
    if "foreground" not in kwargs:
        kwargs["foreground"] = "blue"
    code = f"""
tag_no += 1
tag_name = "custom" + str(tag_no)
idlelib.pyshell.flist.pyshell.text.tag_configure(tag_name, **{str(kwargs)})
idlelib.pyshell.flist.pyshell.text.tag_lower(tag_name)
idlelib.pyshell.flist.pyshell.write({repr(s)}, tag_name)
"""
    hypno.inject_py(pid, code)

def paint_line(background = "", bgstipple = ""):
    code = f"""
tag_no += 1
tag_name = "custom" + str(tag_no)
idlelib.pyshell.flist.pyshell.text.tag_configure(tag_name, background = "{background}", bgstipple = "{bgstipple}")
idlelib.pyshell.flist.pyshell.text.tag_lower(tag_name)
idlelib.pyshell.flist.pyshell.text.mark_set("iomark", "end-1c")
idlelib.pyshell.flist.pyshell.text.insert("iomark", "\\n", tag_name)
idlelib.pyshell.flist.pyshell.text.mark_set("iomark", "end-1c")
idlelib.pyshell.flist.pyshell.text.see("iomark")
idlelib.pyshell.flist.pyshell.text.update()
"""
    hypno.inject_py(pid, code)
   
def clear():
    code = """
idlelib.pyshell.flist.pyshell.text.mark_set("iomark", "1.0")
idlelib.pyshell.flist.pyshell.text.delete("1.0", "end")
"""
    hypno.inject_py(pid, code)




import idle_term
import tqdm
import time


def test1():        
    idle_term.print_fancy("'abcde", end = "\n", foreground="purple", background = "green", font=("Algerian", 14, "bold"))
    idle_term.print_fancy("fghj", foreground="yellow", background = "green", font=("Algerian", 14, "bold"))
    print("a"*20, end = "\r")
    print("b"*15, end = "\n") # on python 3.7.8 remaining "a" letters at the right may sometimes become black, this problem doesnt occur on python 3.13.0
    print("c"*10, end = "\r")
    print("d"*5, end = "\r")
    print("e"*1)
    print("aaaaa\rb\nbbb\rcc", end = "")    
    print("\b")
    idle_term.paint_line(background = "green")
    print()


def test2():
    for i in tqdm.tqdm(range(10**7)):
        pass

        
test1()
